#!/usr/bin/env bash
if [ -z "$BASH" ] ; then
   bash  $0
   exit
fi



my_name=$0


function setup_environment {
  bf=""
  n=""
  ORGANISATION="Univerzita Jana Evangelisty Purkyně v Ústí nad Labem"
  URL="eduroam.ujep.cz"
  SUPPORT="sit@rt.ujep.cz"
if [ ! -z "$DISPLAY" ] ; then
  if which zenity 1>/dev/null 2>&1 ; then
    ZENITY=`which zenity`
  elif which kdialog 1>/dev/null 2>&1 ; then
    KDIALOG=`which kdialog`
  else
    if tty > /dev/null 2>&1 ; then
      if  echo $TERM | grep -E -q "xterm|gnome-terminal|lxterminal"  ; then
        bf="[1m";
        n="[0m";
      fi
    else
      find_xterm
      if [ -n "$XT" ] ; then
        $XT -e $my_name
      fi
    fi
  fi
fi
}

function split_line {
echo $1 | awk  -F '\\\\n' 'END {  for(i=1; i <= NF; i++) print $i }'
}

function find_xterm {
terms="xterm aterm wterm lxterminal rxvt gnome-terminal konsole"
for t in $terms
do
  if which $t > /dev/null 2>&1 ; then
  XT=$t
  break
  fi
done
}


function ask {
     T="eduroam CAT"
#  if ! [ -z "$3" ] ; then
#     T="$T: $3"
#  fi
  if [ ! -z $KDIALOG ] ; then
     if $KDIALOG --yesno "${1}\n${2}?" --title "$T" ; then
       return 0
     else
       return 1
     fi
  fi
  if [ ! -z $ZENITY ] ; then
     text=`echo "${1}" | fmt -w60`
     if $ZENITY --no-wrap --question --text="${text}\n${2}?" --title="$T" 2>/dev/null ; then
       return 0
     else
       return 1
     fi
  fi

  yes=A
  no=N
  yes1=`echo $yes | awk '{ print toupper($0) }'`
  no1=`echo $no | awk '{ print toupper($0) }'`

  if [ $3 == "0" ]; then
    def=$yes
  else
    def=$no
  fi

  echo "";
  while true
  do
  split_line "$1"
  read -p "${bf}$2 ${yes}/${no}? [${def}]:$n " answer
  if [ -z "$answer" ] ; then
    answer=${def}
  fi
  answer=`echo $answer | awk '{ print toupper($0) }'`
  case "$answer" in
    ${yes1})
       return 0
       ;;
    ${no1})
       return 1
       ;;
  esac
  done
}

function alert {
  if [ ! -z $KDIALOG ] ; then
     $KDIALOG --sorry "${1}"
     return
  fi
  if [ ! -z $ZENITY ] ; then
     $ZENITY --warning --text="$1" 2>/dev/null
     return
  fi
  echo "$1"

}

function show_info {
  if [ ! -z $KDIALOG ] ; then
     $KDIALOG --msgbox "${1}"
     return
  fi
  if [ ! -z $ZENITY ] ; then
     $ZENITY --info --width=500 --text="$1" 2>/dev/null
     return
  fi
  split_line "$1"
}

function confirm_exit {
  if [ ! -z $KDIALOG ] ; then
     if $KDIALOG --yesno "Chcete skutečně skončit?"  ; then
     exit 1
     fi
  fi
  if [ ! -z $ZENITY ] ; then
     if $ZENITY --question --text="Chcete skutečně skončit?" 2>/dev/null ; then
        exit 1
     fi
  fi
}



function prompt_nonempty_string {
  prompt=$2
  if [ ! -z $ZENITY ] ; then
    if [ $1 -eq 0 ] ; then
     H="--hide-text "
    fi
    if ! [ -z "$3" ] ; then
     D="--entry-text=$3"
    fi
  elif [ ! -z $KDIALOG ] ; then
    if [ $1 -eq 0 ] ; then
     H="--password"
    else
     H="--inputbox"
    fi
  fi


  out_s="";
  if [ ! -z $ZENITY ] ; then
    while [ ! "$out_s" ] ; do
      out_s=`$ZENITY --entry --width=300 $H $D --text "$prompt" 2>/dev/null`
      if [ $? -ne 0 ] ; then
        confirm_exit
      fi
    done
  elif [ ! -z $KDIALOG ] ; then
    while [ ! "$out_s" ] ; do
      out_s=`$KDIALOG $H "$prompt" "$3"`
      if [ $? -ne 0 ] ; then
        confirm_exit
      fi
    done  
  else
    while [ ! "$out_s" ] ; do
      read -p "${prompt}: " out_s
    done
  fi
  echo "$out_s";
}

function user_cred {
  PASSWORD="a"
  PASSWORD1="b"

  if ! USER_NAME=`prompt_nonempty_string 1 "zadajte své uživatelské jméno"` ; then
    exit 1
  fi

  while [ "$PASSWORD" != "$PASSWORD1" ]
  do
    if ! PASSWORD=`prompt_nonempty_string 0 "zadajte heslo"` ; then
      exit 1
    fi
    if ! PASSWORD1=`prompt_nonempty_string 0 "opakujte zadání hesla"` ; then
      exit 1
    fi
    if [ "$PASSWORD" != "$PASSWORD1" ] ; then
      alert "heslo není shodné"
    fi
  done
}
setup_environment
show_info "Tento instalační program byl připravený pro ${ORGANISATION}\n\nDalší informace a komentáře:\n\nEMAIL: ${SUPPORT}\nWWW: ${URL}\n\nInstalační program byl vytvořený pomocí programu z projektu GEANT."
if ! ask "Tento instalační program bude fungovat pouze pokud jste členem organizace ${bf}Univerzita Jana Evangelisty Purkyně v Ústí nad Labem.${n}" "Pokračovat" 1 ; then exit; fi
if [ -d $HOME/.cat_installer ] ; then
   if ! ask "Adresář $HOME/.cat_installer už existuje, některé soubory v něm budou přepsány." "Pokračovat" 1 ; then exit; fi
else
  mkdir $HOME/.cat_installer
fi
# save certificates
echo "-----BEGIN CERTIFICATE-----
MIIENjCCAx6gAwIBAgIBATANBgkqhkiG9w0BAQUFADBvMQswCQYDVQQGEwJTRTEU
MBIGA1UEChMLQWRkVHJ1c3QgQUIxJjAkBgNVBAsTHUFkZFRydXN0IEV4dGVybmFs
IFRUUCBOZXR3b3JrMSIwIAYDVQQDExlBZGRUcnVzdCBFeHRlcm5hbCBDQSBSb290
MB4XDTAwMDUzMDEwNDgzOFoXDTIwMDUzMDEwNDgzOFowbzELMAkGA1UEBhMCU0Ux
FDASBgNVBAoTC0FkZFRydXN0IEFCMSYwJAYDVQQLEx1BZGRUcnVzdCBFeHRlcm5h
bCBUVFAgTmV0d29yazEiMCAGA1UEAxMZQWRkVHJ1c3QgRXh0ZXJuYWwgQ0EgUm9v
dDCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALf3GjPm8gAELTngTlvt
H7xsD821+iO2zt6bETOXpClMfZOfvUq8k+0DGuOPz+VtUFrWlymUWoCwSXrbLpX9
uMq/NzgtHj6RQa1wVsfwTz/oMp50ysiQVOnGXw94nZpAPA6sYapeFI+eh6FqUNzX
mk6vBbOmcZSccbNQYArHE504B4YCqOmoaSYYkKtMsE8jqzpPhNjfzp/haW+710LX
a0Tkx63ubUFfclpxCDezeWWkWaCUN/cALw3CknLa0Dhy2xSoRcRdKn23tNbE7qzN
E0S3ySvdQwAl+mG5aWpYIxG3pzOPVnVZ9c0p10a3CitlttNCbxWyuHv77+ldU9U0
WicCAwEAAaOB3DCB2TAdBgNVHQ4EFgQUrb2YejS0Jvf6xCZU7wO94CTLVBowCwYD
VR0PBAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wgZkGA1UdIwSBkTCBjoAUrb2YejS0
Jvf6xCZU7wO94CTLVBqhc6RxMG8xCzAJBgNVBAYTAlNFMRQwEgYDVQQKEwtBZGRU
cnVzdCBBQjEmMCQGA1UECxMdQWRkVHJ1c3QgRXh0ZXJuYWwgVFRQIE5ldHdvcmsx
IjAgBgNVBAMTGUFkZFRydXN0IEV4dGVybmFsIENBIFJvb3SCAQEwDQYJKoZIhvcN
AQEFBQADggEBALCb4IUlwtYj4g+WBpKdQZic2YR5gdkeWxQHIzZlj7DYd7usQWxH
YINRsPkyPef89iYTx4AWpb9a/IfPeHmJIZriTAcKhjW88t5RxNKWt9x+Tu5w/Rw5
6wwCURQtjr0W4MHfRnXnJK3s9EK0hZNwEGe6nQY1ShjTK3rMUUKhemPR5ruhxSvC
Nr4TDea9Y355e6cJDUCrat2PisP29owaQgVR1EX1n6diIWgVIEM8med8vSTYqZEX
c4g/VhsxOBi0cQ+azcgOno4uG+GMmIPLHzHxREzGBHNJdmAPx/i9F4BrLunMTA5a
mnkPIAou1Z5jJh5VkpTYghdae9C8x49OhgQ=
-----END CERTIFICATE-----

-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----

" > $HOME/.cat_installer/ca.pem
function run_python_script {
PASSWORD=$( echo "$PASSWORD" | sed "s/'/\\\'/g" )
if python << EEE1 > /dev/null 2>&1
import dbus
EEE1
then
    PYTHON=python
elif python3 << EEE2 > /dev/null 2>&1
import dbus
EEE2
then
    PYTHON=python3
else
    PYTHON=none
    return 1
fi

$PYTHON << EOF > /dev/null 2>&1
#-*- coding: utf-8 -*-
import dbus
import re
import sys
import uuid
import os

class EduroamNMConfigTool:

    def connect_to_NM(self):
        #connect to DBus
        try:
            self.bus = dbus.SystemBus()
        except dbus.exceptions.DBusException:
            print("Can't connect to DBus")
            sys.exit(2)
        #main service name
        self.system_service_name = "org.freedesktop.NetworkManager"
        #check NM version
        self.check_nm_version()
        if self.nm_version == "0.9" or self.nm_version == "1.0":
            self.settings_service_name = self.system_service_name
            self.connection_interface_name = "org.freedesktop.NetworkManager.Settings.Connection"
            #settings proxy
            sysproxy = self.bus.get_object(self.settings_service_name, "/org/freedesktop/NetworkManager/Settings")
            #settings intrface
            self.settings = dbus.Interface(sysproxy, "org.freedesktop.NetworkManager.Settings")
        elif self.nm_version == "0.8":
            #self.settings_service_name = "org.freedesktop.NetworkManagerUserSettings"
            self.settings_service_name = "org.freedesktop.NetworkManager"
            self.connection_interface_name = "org.freedesktop.NetworkManagerSettings.Connection"
            #settings proxy
            sysproxy = self.bus.get_object(self.settings_service_name, "/org/freedesktop/NetworkManagerSettings")
            #settings intrface
            self.settings = dbus.Interface(sysproxy, "org.freedesktop.NetworkManagerSettings")
        else:
            print("This Network Manager version is not supported")
            sys.exit(2)

    def check_opts(self):
        self.cacert_file = '${HOME}/.cat_installer/ca.pem'
        self.pfx_file = '${HOME}/.cat_installer/user.p12'
        if not os.path.isfile(self.cacert_file):
            print("Certificate file not found, looks like a CAT error")
            sys.exit(2)

    def check_nm_version(self):
        try:
            proxy = self.bus.get_object(self.system_service_name, "/org/freedesktop/NetworkManager")
            props = dbus.Interface(proxy, "org.freedesktop.DBus.Properties")
            version = props.Get("org.freedesktop.NetworkManager", "Version")
        except dbus.exceptions.DBusException:
            version = "0.8"
        if re.match(r'^1\.', version):
            self.nm_version = "1.0"
            return
        if re.match(r'^0\.9', version):
            self.nm_version = "0.9"
            return
        if re.match(r'^0\.8', version):
            self.nm_version = "0.8"
            return
        else:
            self.nm_version = "Unknown version"
            return

    def byte_to_string(self, barray):
        return "".join([chr(x) for x in barray])


    def delete_existing_connections(self, ssid):
        "checks and deletes earlier connections"
        try:
            conns = self.settings.ListConnections()
        except dbus.exceptions.DBusException:
            print("DBus connection problem, a sudo might help")
            exit(3)
        for each in conns:
            con_proxy = self.bus.get_object(self.system_service_name, each)
            connection = dbus.Interface(con_proxy, "org.freedesktop.NetworkManager.Settings.Connection")
            try:
               connection_settings = connection.GetSettings()
               if connection_settings['connection']['type'] == '802-11-wireless':
                   conn_ssid = self.byte_to_string(connection_settings['802-11-wireless']['ssid'])
                   if conn_ssid == ssid:
                       connection.Delete()
            except dbus.exceptions.DBusException:
               pass

    def add_connection(self,ssid):
        server_alt_subject_name_list = dbus.Array({'DNS:radius.ujep.cz'})
        server_name = 'radius.ujep.cz'
        if self.nm_version == "0.9" or self.nm_version == "1.0":
             match_key = 'altsubject-matches'
             match_value = server_alt_subject_name_list
        else:
             match_key = 'subject-match'
             match_value = server_name
            
        s_con = dbus.Dictionary({
            'type': '802-11-wireless',
            'uuid': str(uuid.uuid4()),
            'permissions': ['user:$USER'],
            'id': ssid 
        })
        s_wifi = dbus.Dictionary({
            'ssid': dbus.ByteArray(ssid.encode('utf8')),
            'security': '802-11-wireless-security'
        })
        s_wsec = dbus.Dictionary({
            'key-mgmt': 'wpa-eap',
            'proto': ['rsn',],
            'pairwise': ['ccmp',],
            'group': ['ccmp', 'tkip']
        })
        s_8021x = dbus.Dictionary({
            'eap': ['peap'],
            'identity': '$USER_NAME',
            'ca-cert': dbus.ByteArray("file://{0}\0".format(self.cacert_file).encode('utf8')),
             match_key: match_value,
            'password': '$PASSWORD',
            'phase2-auth': 'mschapv2',
        })
        s_ip4 = dbus.Dictionary({'method': 'auto'})
        s_ip6 = dbus.Dictionary({'method': 'auto'})
        con = dbus.Dictionary({
            'connection': s_con,
            '802-11-wireless': s_wifi,
            '802-11-wireless-security': s_wsec,
            '802-1x': s_8021x,
            'ipv4': s_ip4,
            'ipv6': s_ip6
        })
        self.settings.AddConnection(con)

    def main(self):
        self.check_opts()
        ver = self.connect_to_NM()
        self.delete_existing_connections('eduroam')
        self.add_connection('eduroam')

if __name__ == "__main__":
    ENMCT = EduroamNMConfigTool()
    ENMCT.main()
EOF
}
function create_wpa_conf {
cat << EOFW >> $HOME/.cat_installer/cat_installer.conf

network={
  ssid="eduroam"
  key_mgmt=WPA-EAP
  pairwise=CCMP
  group=CCMP TKIP
  eap=PEAP
  ca_cert="${HOME}/.cat_installer/ca.pem"
  identity="${USER_NAME}"
  domain_suffix_match="radius.ujep.cz"
  phase2="auth=MSCHAPV2"
  password="${PASSWORD}"
}
EOFW
chmod 600 $HOME/.cat_installer/cat_installer.conf
}
#prompt user for credentials
  user_cred
  if run_python_script ; then
   show_info "Instalace proběhla úspěšně"
else
   show_info "Konfigurace Network Manageru nebyla úspěšná. Bude vytvořen soubor wpa_supplicant.conf"
   if ! ask "Network Manager configuration failed, but we may generate a wpa_supplicant configuration file if you wish. Be warned that your connection password will be saved in this file as clear text." "Zapsat soubor" 1 ; then exit ; fi

if [ -f $HOME/.cat_installer/cat_installer.conf ] ; then
  if ! ask "Soubor $HOME/.cat_installer/cat_installer.conf již existuje, bude přepsán." "Pokračovat" 1 ; then confirm_exit; fi
  rm $HOME/.cat_installer/cat_installer.conf
  fi
   create_wpa_conf
   show_info "Výstup je uložený do $HOME/.cat_installer/cat_installer.conf"
fi
